/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */
/*
 * PrototypeFactoryUnitTest.cpp
 *
 *  Created on: Jan 03, 2011
 *      Author: heber
 */
// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include "CodePatterns/Assert.hpp"

#include "PrototypeFactoryUnitTest.hpp"

#include "stubs/CommonStub.hpp"
#include "stubs/PrototypeFactoryStub.hpp"

#include <typeinfo>

#ifdef HAVE_TESTRUNNER
#include "UnitTestMain.hpp"
#endif /*HAVE_TESTRUNNER*/

/********************************************** Test classes **************************************/

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( PrototypeFactoryTest );

void PrototypeFactoryTest::setUp()
{
  rndA =
      PrototypeFactoryStub::getInstance().
      PrototypeTable[PrototypeFactoryStub::Aclass]->clone();
  CPPUNIT_ASSERT_EQUAL( 0,  rndA->getcount() );
  rndB =
      PrototypeFactoryStub::getInstance().
      PrototypeTable[PrototypeFactoryStub::Bclass]->clone();
  CPPUNIT_ASSERT_EQUAL( 0,  rndB->getcount() );

  rndA_1 = NULL;
  rndA_2 = NULL;
  rndA_3 = NULL;
}

void PrototypeFactoryTest::tearDown()
{
  delete rndA;
  delete rndA_1;
  delete rndA_2;
  delete rndA_3;
  delete rndB;
  PrototypeFactoryStub::purgeInstance();
}

void PrototypeFactoryTest::DistributionTest()
{
  // check the injectiveness of enum and string map
  for (PrototypeFactoryStub::NameMap::const_iterator
      iter = PrototypeFactoryStub::getInstance().names.begin();
      iter != PrototypeFactoryStub::getInstance().names.end();
      ++iter) {
    CPPUNIT_ASSERT_EQUAL(
        iter->second,
        PrototypeFactoryStub::getInstance().getName(
            PrototypeFactoryStub::getInstance().getEnum(
                iter->second
            )
        )
    );
  }

  // check distributions in the table
  CPPUNIT_ASSERT_EQUAL(
      std::string(typeid(Prototype<teststubs::Aclass>).name()),
      std::string(typeid(*rndA).name())
  );
  CPPUNIT_ASSERT_EQUAL(
      std::string(typeid(Prototype<teststubs::Bclass>).name()),
      std::string(typeid(*rndB).name())
  );
}

void PrototypeFactoryTest::getProductEnumTest()
{
  rndA_1 =
      PrototypeFactoryStub::getInstance().getProduct(PrototypeFactoryStub::Aclass);
  CPPUNIT_ASSERT( typeid(*rndA) == typeid(*rndA_1) );
}

void PrototypeFactoryTest::getProductNameTest()
{
  rndA_1 =
      PrototypeFactoryStub::getInstance().getProduct(std::string("Aclass"));
  CPPUNIT_ASSERT( typeid(*rndA) == typeid(*rndA_1) );
}

void PrototypeFactoryTest::getProductTypeTest()
{
  rndA_1 =
      PrototypeFactoryStub::getInstance().getProduct( typeid(teststubs::Aclass) );
  CPPUNIT_ASSERT( typeid(*rndA) == typeid(*rndA_1) );
}

void PrototypeFactoryTest::Individualitytest()
{
  // increasing this does not change the prototype
  rndA->count();

  rndA_1 = PrototypeFactoryStub::getInstance().getProduct(PrototypeFactoryStub::Aclass);
  CPPUNIT_ASSERT_EQUAL( 0,  rndA_1->getcount() );
  CPPUNIT_ASSERT( rndA->getcount() != rndA_1->getcount() );
  rndA_1->count();
  CPPUNIT_ASSERT( rndA->getcount() == rndA_1->getcount() );

  rndA_2 = PrototypeFactoryStub::getInstance().getProduct(PrototypeFactoryStub::Aclass);
  CPPUNIT_ASSERT_EQUAL( 0,  rndA_2->getcount() );
  CPPUNIT_ASSERT( rndA->getcount() != rndA_2->getcount() );
  rndA_2->count();
  CPPUNIT_ASSERT( rndA->getcount() == rndA_2->getcount() );
  CPPUNIT_ASSERT( rndA_1->getcount() == rndA_2->getcount() );
}

void PrototypeFactoryTest::getPrototypetest()
{
  // this method is protected and only friends may access it.
  const IPrototype& rndA_1c = PrototypeFactoryStub::getInstance().getPrototype(std::string("Aclass"));

  // clone the type and check whether new default values holds
  rndA_2 = PrototypeFactoryStub::getInstance().getProduct(PrototypeFactoryStub::Aclass);
  CPPUNIT_ASSERT_EQUAL( rndA_1c.getcount(), rndA_2->getcount() );
  rndA_2->count();
  CPPUNIT_ASSERT( rndA_1c.getcount() != rndA_2->getcount() );

  // the following is not possible
  //rndA_1c.count();

  rndA_3 = PrototypeFactoryStub::getInstance().getProduct(PrototypeFactoryStub::Aclass);
  CPPUNIT_ASSERT_EQUAL( 0, rndA_3->getcount() );
  CPPUNIT_ASSERT( rndA->getcount() == rndA_3->getcount() );
}
